function [sol,stats] = idetest_70(problem,atol,rtol,plotsel,opt)  
% Solve a problem in the testset format using ODE15I solver  
% The ODE15I solver is available in MATLAB 7.0 and above.  
%   
% [sol,stats] = idetest_70(problem,atol,rtol,plotsel,opt)  
%   
% INPUT  
% problem = string with the name of the IVP TEST SET problem  
% ODE problems  
%     Hires, Pollu, Ringmod, Medakzo, Emep, Plei, vdpol,  
%     Rober, Orego, E5  
% DAE problems  
%     Chemakzo, Andrews, Transamp, Pump, Tba, Caraxis  
%     Fekete, Crank, Water  
% IDE problems  
%     Nand, Wheel  
%  
% atol = absolute tolerance  
% rtol = relative tolerance  
% plotsel = index of component to be plotted (optional)   
% opt = solver parameters set with ODESET (optional) 
%  
% OUTPUT  
% sol.x  : mesh point of the approximate solution  
% sol.y  : solution computed at sol.x  
% sol.solver : solver used  
% sol.scd : significant digit of the approximate solution  
% sol.mescd : mixed significant digit of the approximate solution  
% sol.ierr  : error flag (0 = successfull run, 1 = error) 
% 
% stats.nsteps : number of successfull steps  
% stats.nfailed : number of failed steps  
% stats.nfevals : number of function evaluation  
% stats.npds : number of partial derivative  
% stats.ndecomps : number of LU decomposition  
% stats.nsolves : number of solution of linear systems  
%   
%---------------------------------------------------------------------  
%     This file is part of the Test Set for IVP solvers  
%     http://www.dm.uniba.it/~testset/  
%  
%  
%     DISCLAIMER: see  
%     http://www.dm.uniba.it/~testset/disclaimer.php  
%  
%     The most recent version of this source file can be found at  
%     http://www.dm.uniba.it/~testset/matlab/idetest_70.m  
%  
%     This is revision: 
%     
% $Id: idetest_70.m,v 1.2 2008/02/08 11:09:39 testset Exp $  
%-----------------------------------------------------------------------  
    
if nargin < 3  
  error('Not enough input arguments')  
end    
solver = 'ode15i';  
sol = [];  
stats = [];  
  
% Extract problem data  
mproblem = minterface(problem);  
data = mproblem('Prob');  
  
% Adjust tolerances  
[atolv,rtolv] = mproblem('Tolerance',data.neqn,atol,rtol);  
if any(atolv ~= atol)  
  warning('Absolute tolerance have been changed');  
end  
if any(rtolv ~= rtol)   
  error('Relative tolerance must be a scalar');     
end  
  
% IDE function  
switch data.type  
    
 case 'ODE'  
  y0  = mproblem('Init',data.neqn,data.t(1));  
  yp0 = mproblem('Feval',data.neqn,data.t(1),y0,[],[],[]);  
  ide = @(t,y,yp) yp - mproblem('Feval',data.neqn,t,y,[],[],[]);  
  M0 = speye(data.neqn);  
 case {'DAE','IDE'}  
  if any(data.ind > 1)  
    error('Index %i reported. %s cannot solve DAEs of index > 1.', ...  
          max(data.ind),upper(solver));  
  end  
  [y0,yp0] = mproblem('Init',data.neqn,data.t(1));    
  switch data.type  
   case 'IDE'  
    ide = @(t,y,yp) mproblem('Feval',data.neqn,t,y,yp,[],[]);  
   case 'DAE'  
    t0 = data.t(1);  
    M0 = odemass(t0,y0,yp0);  % Constant mass matrix  
    ide = @(t,y,yp) M0*yp - mproblem('Feval',data.neqn,t,y,yp,[],[]);  
  end    
    
end % IDE function  
  
% Set solver options  
if nargin < 5 || ~isstruct(opt) 
  opt = []; 
end 
opt = odeset(opt,'Stats','on'); 
opt = odeset(opt,'Reltol',rtol,'Abstol',atolv);  
opt = odeset(opt,'Jacobian',@jac); % could return empty  
if nargin < 4 
  plotsel = []; 
end 
if ~isempty(plotsel) 
  opt = odeset(opt,'OutputFcn',@odeplot,'OutputSel',plotsel);  
end  
  
% Print test info  
fprintf('%s \n\n',data.fullnm);  
fprintf('Solver: %s \n\n', upper(solver));  
fprintf('%e absolute tolerance \n', atol);  
fprintf('%e relative tolerance \n\n', rtol);  
  
% First call to the solver  
tstart = data.t(1);  
if data.ndisc == 0  
  tfinal = data.t(end); % Hit the end-point exactly  
else  
  tfinal = data.t(2) - eps(data.t(2)); % Avoid discontinuity  
end    
sol = feval(solver,ide,[tstart,tfinal],y0,yp0,opt);  
if sol.x(end) < tfinal    
  sol.ierr = 1; 
  return;  % did not finish successfully...  
end  
  
% Handle discontinuities  
printstats = strcmp(odeget(opt,'Stats','off'),'on'); 
for i = 3 : length(data.t)  
  if printstats 
    fprintf('\n'); 
  end   
  tstart = data.t(i-1) + eps(data.t(i-1));  
  tfinal = data.t(i)   - eps(data.t(i));   
  if i == length(data.t)  
    tfinal = data.t(end);  % Hit the end-point exactly  
  end    
  sol.x(end) = tstart;   % Move accross the discontinuity  
  sol = odextend(sol,ide,tfinal);      
  if sol.x(end) < tfinal  
    sol.ierr = 1;   
    return;  % did not finish successfully...  
  end  
end  
  
if sol.x(end) == data.t(end)    % Successful run   
    
  % Adjust the plot, if needed  
  if ~isempty(plotsel) && (data.ndisc > 0)  
    plot(sol.x,sol.y(plotsel,:),'-o');  
  end    
    
  % Extract stats  
  stats = sol.stats;  
    
  % Extract reference solution and compute actual errors  
  sol.atol = atolv;  
  sol.rtol = rtolv;  
  
  yref = mproblem('Solut',data.neqn,data.t(end));   
  yfinal = sol.y(:,end);  
  tolvec = (length(atolv) > 1);  
  [mescd,scd] = mproblem('Report',data.neqn, ...  
                  yref,yfinal,problem,tolvec,atolv,rtolv);  
          
  fprintf('\n');                  
  fprintf('mescd = %e \n', mescd);  
  fprintf('scd   = %e \n \n', scd);  
    
  sol.scd = max(scd);  
  sol.mescd = max(mescd); 
  sol.ierr  = 0; 
else 
  sol.ierr  = 1;    
end  
  
% ==========================================================================  
% Nested functions  
%   
  
  function M = odemass(t,y,yp)    
    if data.nummas && ~isequal(data.type,'DAE')  
      M = [];  
    else    
      m = data.neqn;    
      mlmas = data.mlmas;  
      mumas = data.mumas;    
      if mlmas == m  
        % Full storage  
        M = mproblem('Meval',m,m,t,y,yp,[],[]);  
      else    
        % Band storage  
        ldim = mlmas + mumas + 1;  
        M = mproblem('Meval',ldim,m,t,y,yp,[],[]);  
        M = spdiags(M',[mumas:-1:-mlmas],m,m);  
      end        
    end  
  end  % odemass    
    
  % -----------------------------------      
  
  function J = odejac(t,y,yp)    
    if data.numjac  
      J = [];  
    else    
      m = data.neqn;    
      mljac = data.mljac;  
      mujac = data.mujac;    
      if mljac == m  
        % Full storage  
        J = mproblem('Jeval',m,m,t,y,yp,[],[]);  
      else    
        % Band storage  
        ldim = mljac + mujac + 1;  
        J = mproblem('Jeval',ldim,m,t,y,yp,[],[]);  
        J = spdiags(J',[mujac:-1:-mljac],m,m);  
      end        
    end      
  end  % odejac  
    
  % -----------------------------------      
  
  function [J,M] = jac(t,y,yp)  
    J = -odejac(t,y,yp);   % M*yp - f(t,y)  
    if isequal(data.type,'IDE')  
      M = odemass(t,y,yp);  
    else   
      % constant mass matrix, already available  
      M = M0;  
    end  
  end  % jac  
    
  % -----------------------------------      
    
end  % idetest_70  
  
%EOF [idetest_70.m]  
